from __future__ import unicode_literals

from datetime import datetime
from unittest import TestCase

from mock import Mock, patch

from repos import errors, launchpad


class FakeMP(object):
    """Fake launchpadlib Merge Proposal object."""

    def __init__(self, status, commit_message=None):
        self.queue_status = status
        self.date_created = datetime.now()
        self.commit_message = commit_message
        self.source_branch = Mock(bzr_identity='source')
        self.target_branch = Mock(bzr_identity='target')
        self.reviewer = Mock(name='reviewer')


class FakeProject(object):
    """Fake launchpadlib Project object."""

    def __init__(self, with_approved_mp=False):
        self.with_approved_mp = with_approved_mp

    def getMergeProposals(self):
        proposals = [FakeMP(status='merged', commit_message='merged mp')]
        if self.with_approved_mp:
            proposals.append(
                FakeMP(status='approved', commit_message='approved mp'))
        return proposals


class LaunchpadMock(object):
    """Fake launchpadlib Launchpad service object."""

    def __init__(self, username, env, cache_dir):
        self.username = username
        self.env = env
        self.cache_dir = cache_dir
        self.projects = {
            'test-project': FakeProject(with_approved_mp=True),
            'another-project': FakeProject(),
        }


class LaunchpadServiceTestCase(TestCase):

    def setUp(self):
        super(LaunchpadServiceTestCase, self).setUp()
        self.mock_lp_login = patch(
            'launchpadlib.launchpad.Launchpad.login_anonymously',
            LaunchpadMock)
        self.mock_lp_login.start()
        self.addCleanup(self.mock_lp_login.stop)
        self.lp = launchpad.Service()

    def test_launchpad_login(self):
        self.assertEqual(self.lp.launchpad.username, 'locolander')
        self.assertEqual(self.lp.launchpad.env, 'production')
        self.assertEqual(self.lp.launchpad.cache_dir, launchpad.CACHE_DIR)

    def test_project_not_match(self):
        with self.assertRaises(errors.RepositoryDoesNotExist):
            self.lp.approved_requests_for_project_url('not-found')

    def test_project_not_found(self):
        with self.assertRaises(errors.RepositoryDoesNotExist):
            self.lp.approved_requests_for_project_url('lp:not-found')

    def test_valid_project_without_approved_mp(self):
        mps = self.lp.approved_requests_for_project_url('lp:test-project')
        self.assertEqual(len(mps), 1)
        mp = mps[0]
        self.assertIsInstance(mp, dict)

        expected_keys = ['date_created', 'source', 'target', 'reviewers',
                         'commit_message']
        for key in expected_keys:
            self.assertIn(key, mp)
        self.assertEqual(mp['commit_message'], 'approved mp')

    def test_valid_project_return_approved_mp(self):
        mps = self.lp.approved_requests_for_project_url('lp:test-project')
        self.assertEqual(len(mps), 1)
        mp = mps[0]
        self.assertIsInstance(mp, dict)

        expected_keys = ['date_created', 'source', 'target', 'reviewers',
                         'commit_message']
        for key in expected_keys:
            self.assertIn(key, mp)
        self.assertEqual(mp['commit_message'], 'approved mp')
